from vinca._config import dolt_repo

def run():
    if dolt_repo and dolt_repo.days_since_last_sync > 1:
            dolt_repo.ask_and_merge()

    try:
        from fire import Fire
        from vinca import _cli_objects
        Fire(component=_cli_objects, name='vinca')
    finally:
        if dolt_repo:
            dolt_repo.commit()

    




import datetime

def unixepoch_local():
        return (datetime.datetime.now() - datetime.datetime(year=1970, month=1, day=1)).total_seconds()

def now():
        return unixepoch_local() / 86400

def today():
        today = int(now())
        assert today == (datetime.date.today() - datetime.date(year=1970,month=1,day=1)).days
        return today

class JulianDate(float):
        
        def __str__(self):
            date = datetime.date(year=1970,month=1,day=1) + datetime.timedelta(days = 1) * int(self)
            return f'{date}     write this on the command line as {int(self) - today()}'




# commit at the end of every session (if diff says to do so)
# if it has been more than a day fetch to determine what to do next
# handle no internet

from pathlib import Path as _Path
from subprocess import run
import pymysql
import datetime
TODAY = datetime.date.today()
DAY = datetime.datetime(days = 1)

class Dolt_Repo:

    def __init__(self, dolt_repo_path):
            self.path = Path(dolt_repo_path).expanduser()
            assert path.exists()
            self.cursor = self.launch_server()

    def init_dolt_repo(self):
            raise NotImplementedError

    def create_dolt_remote_dolthub(self):
            raise NotImplementedError

    def launch_server(self): 
            # locally run a mysql server
           run(['dolt','sql-server', '--multi-db-dir',self.path.parent])
           cursor = pymysql.connect(host="127.0.0.1",port=3306,user="root",db=self.path.name)
           return cursor

    def commit(self):
            self.cursor.execute('-a','-m','normal study')

    def ancestors_log(self, commit):
            'get all the ancestors of a commit, excluding merged branches'
            # select parent_index=0 excludes merged ancestors
            log = [commit]
            while log[-1] is not None:
                    self.cursor.execute('select parent_hash from dolt_commit_ancestors '
                    'where parent_index = 0 and commit_hash = ?', (log[-1],))
                    parent = self.cursor.fetchone()[0]
                    log.append(parent)
            return log

    def common_ancestor(self, commit_1, commit_2):
            # find the mergebase of two commits
            ancestors_1 = self.ancestors_log(commit_1)
            ancestors_2 = self.ancestors_log(commit_2)
            common_ancestors = [a for a in ancestors_1 if a in ancestors_2]
            assert common_ancestors, f"commits {commit_1} and {commit_2} are not cousins!"
            return common_ancestors[0]

    def fetch_origin(self):
            self.cursor.execute("call dolt_fetch('origin','main')")

    @property
    def head(self):
            self.execute("select hashof('main')")
            return self.cursor.fetchone()[0]

    @property
    def origin_head(self):
            'hash of last origin commit. remember that origin/main needs to be fetched first'
            self.execute("select hashof('origin/main')")
            return self.cursor.fetchone()[0]

    @property
    def days_since_sync(self):
            return (TODAY - self.last_sync_date) / DAY

    @property
    def days_since_fetch(self):
            return (TODAY - self.commit_date(self.origin_head)) / DAY

    def commit_date(self, commit):
            'return the date of the most recent commit'
            self.cursor.execute('select date from dolt_commits where commit = ?', ?)
            return self.cursor.fetchone()[0]

    @property
    def last_sync_commit(self):
            return self.common_ancestor(self.head, self.origin_head)
            
    @property
    def last_sync_date(self):
            return self.commit_date(self.last_sync_commit)

    @propery
    def synchronized(self):
            ' this requires a fetch '
            self.fetch()
            return self.last_sync_commit == self.head


    def push(self):
            self.cursor.execute('call dolt_push();')

    def merge(self):
            self.cursor.execute('call dolt_merge();')

    def sync(self):
            'shorthand for merge and push'
            self.merge()
            self.push()

    def local_unsynced_steps(self):
            return self.ancestors_log(self.head).index(self.last_sync_commit)

    def origin_unsynced_steps(self):
            return self.ancestors_log(self.origin_head).index(self.last_sync_commit)

    def ask_and_sync(self):
            self.fetch()
            if self.local_unsynced_steps == self.origin_unsynced_steps == 0:
                    print('Synchronized. Everything is up to date.')
                    return
            print f'local machine is ahead by {self.local_unsynced_steps} \n'
                  f'online server is ahead by {self.origin_unsynced_steps}\n'
            permission = input('Would you like to sync? Y/n\n')
            if permission in ('\n','y'):
                    self.sync()
                    assert self.synchronized
                    print('Synchronized. Everything is up to date.')

    def status(self):
            self.fetch()
            if self.local_unsynced_steps == self.origin_unsynced_steps == 0:
                    print('Synchronized. Everything is up to date.')
                    return
            print f'local machine is ahead by {self.local_unsynced_steps} \n'
                  f'online server is ahead by {self.origin_unsynced_steps}\n'



# A simple module for reading a keystroke from the user
# adapted from the getchar function in the click library

import os
from types import SimpleNamespace

keys = SimpleNamespace( UP = '\x1b[A', DOWN = '\x1b[B', RIGHT = '\x1b[C', LEFT = '\x1b[D',
                        CTRL_UP = '\x1b[1;5A', CTRL_DOWN = '\x1b[1;5B',
                        CTRL_RIGHT = '\x1b[1;5C', CTRL_LEFT = '\x1b[1;5D',
                        ESC = '\x1b', BACK = '\x7f', CTRL_R = '\x12', CTRL_K = '\x0b')

if os.name == 'posix':
    import tty
    import termios
    import sys
    import contextlib
    @contextlib.contextmanager
    def raw_terminal():
            fd = sys.stdin.fileno()

            old_settings = termios.tcgetattr(fd) # save state of terminal

            try:
                    # the tty has two modes:
                    # 'cooked' waits for the user to press enter
                    # 'raw' passes the typed characters immediately
                    tty.setraw(fd)
                    yield fd
            finally:
                    # restore state of terminal
                    termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
                    sys.stdout.flush() 

    def readkey():
            with raw_terminal() as fd:
                    # a single key can be represented by up to 6 bytes
                    return os.read(fd, 6).decode()
else: # Windows OS
    from msvcrt import getch
    def readkey():
            return getch().decode()



# location of the database file
collection_path = '~/cards.db'

# cards with hidden_tags can only be accessed using `vinca -a ...`
# the option flag -a stands for "all cards"
hidden_tags = ['private',]

# integrate vinca with dolthub for easy database backup across multiple devices
# if this is None we will use the "collection_path" variable instead
dolt_repo_path = None




















# DO NOT EDIT
# ----------------------------------------------

if dolt_repo_path:
        from vinca import _dolt
dolt_repo = None if not dolt_repo_path else _dolt.Dolt_Repo(dolt_repo_path)
